@@ -351,6 +351,10 @@ void handle_ncp (struct sockaddr_ipx *source,
 
 		switch(rq->function)
 		{
+		case 18:
+			printf("fn: %-3d\n", rq->function);
+			printf("Get Volume Info with Number\n");
+			break;
 		case 20:
 			printf("fn: %-3d\n", rq->function);
 			printf("Get File Server Date and Time\n");
@@ -381,6 +385,9 @@ void handle_ncp (struct sockaddr_ipx *source,
 			case 01:
 				printf("Get Directory Path\n");
 				break;
+			case 02:
+				printf("Scan Directory Information\n");
+				break;
 			case 03:
 				printf("Get Effective Directory Rights\n");
 				break;
@@ -390,6 +397,9 @@ void handle_ncp (struct sockaddr_ipx *source,
 			case 06:
 				printf("Get Volume Name\n");
 				break;
+			case 10:
+				printf("Create directory\n");
+				break;
 			case 18:
 				printf("Allocate Permanent Dir Handle\n");
 				break;
@@ -399,6 +409,9 @@ void handle_ncp (struct sockaddr_ipx *source,
 			case 21:
 				printf("Get Volume Info with handle\n");
 				break;
+			case 39:
+				printf("Add ext. Trustee to Dir or File\n");
+				break;
 			case 48:
 				printf("Get Name Space Directory Entry\n");
 				break;
@@ -426,21 +439,48 @@ void handle_ncp (struct sockaddr_ipx *source,
 			case 28:
 				printf("Get Connection Information\n");
 				break;
+			case 50:
+				printf("Create Bindery Object\n");
+				break;
 			case 53:
 				printf("Get Bindery Object ID\n");
 				break;
+			case 54:
+				printf("Get Bindery Object Name\n");
+				break;
 			case 55:
 				printf("Scan Bindery Object\n");
 				break;
+			case 57:
+				printf("Create Property\n");
+				break;
+			case 59:
+				printf("Change Property Security\n");
+				break;
+			case 60:
+				printf("Scan Property\n");
+				break;
 			case 61:
 				printf("Read Property Value\n");
 				break;
 			case 62:
 				printf("Write Property Value\n");
 				break;
+			case 65:
+				printf("Add Bindery Object to Set\n");
+				break;
+			case 67:
+				printf("Is Bindery Object in Set\n");
+				break;
 			case 70:
 				printf("Get Bindery Access Level\n");
 				break;
+			case 72:
+				printf("Get Bindery Object Access Level\n");
+				break;
+			case 75:
+				printf("Keyed change password\n");
+				break;
 			}
 			
 			data += 3;
@@ -476,6 +516,10 @@ void handle_ncp (struct sockaddr_ipx *source,
 			printf("fn: %-3d\n", rq->function);
 			printf("Close File\n");
 			break;
+		case 67:
+			printf("fn: %-3d\n", rq->function);
+			printf("Create File\n");
+			break;
 		case 72:
 			printf("fn: %-3d\n", rq->function);
 			printf("Read from File\n");
@@ -488,6 +532,11 @@ void handle_ncp (struct sockaddr_ipx *source,
 			printf("fn: %-3d\n", rq->function);
 			printf("Set File Time Date Stamp\n");
 			break;
+		case 76:
+			printf("fn: %-3d\n", rq->function);
+			printf("Open File (old)\n");
+			break;
+			
 		case 87:
 			printf("fn: %-3d, subfn: %-3d\n",
 			       rq->function, data[0]);
