@@ -97,6 +97,20 @@ str_lower(char *name)
 	}
 }
 
+static inline int
+ncp_namespace(struct inode *i)
+{
+	struct ncp_server *server   = NCP_SERVER(i);
+	struct nw_info_struct *info = NCP_ISTRUCT(i);
+	return server->name_space[info->volNumber];
+}
+
+static inline int
+ncp_preserve_case(struct inode *i)
+{
+	return (ncp_namespace(i) == NW_NS_OS2);
+}
+
 static struct file_operations ncp_dir_operations = {
         NULL,			/* lseek - default */
 	ncp_dir_read,		/* read - bad */
@@ -133,7 +147,7 @@ struct inode_operations ncp_dir_inode_operations = {
 /* Here we encapsulate the inode number handling that depends upon the
  * mount mode: When we mount a complete server, the memory address of
  * the npc_inode_info is used as an inode. When only a single volume
- * is mounted, then the DosDirNum is used as the inode number. As this
+ * is mounted, then the dirEntNum is used as the inode number. As this
  * is unique for the complete volume, this should enable the NFS
  * exportability of a ncpfs-mounted volume.
  */
@@ -148,7 +162,7 @@ inline ino_t
 ncp_info_ino(struct ncp_server *server, struct ncp_inode_info *info)
 {
 	return ncp_single_volume(server)
-		? info->finfo.i.DosDirNum : (ino_t)info;
+		? info->finfo.i.dirEntNum : (ino_t)info;
 }
 
 static inline int
@@ -351,9 +365,12 @@ ncp_readdir(struct inode *inode, struct file *filp,
                         c_last_returned_index = 0;
                         index = 0;
 
-			for (i = 0; i < c_size; i++)
+			if (!ncp_preserve_case(inode))
 			{
-				str_lower(c_entry[i].i.entryName);
+				for (i = 0; i < c_size; i++)
+				{
+					str_lower(c_entry[i].i.entryName);
+				}
 			}
 		}
 	}
@@ -375,7 +392,7 @@ ncp_readdir(struct inode *inode, struct file *filp,
 
 		if (ncp_single_volume(server))
 		{
-			ino = (ino_t)(entry->i.DosDirNum);
+			ino = (ino_t)(entry->i.dirEntNum);
 		}
 		else
 		{
@@ -671,7 +688,7 @@ ncp_init_root(struct ncp_server *server)
         root->finfo.opened = 0;
 	i->attributes  = aDIR;
 	i->dataStreamSize = 1024;
-	i->DosDirNum = 0;
+	i->dirEntNum = i->DosDirNum = 0;
 	i->volNumber = NCP_NUMBER_OF_VOLUMES+1;	/* illegal volnum */
 	ncp_date_unix2dos(0, &(i->creationTime), &(i->creationDate));
 	ncp_date_unix2dos(0, &(i->modifyTime), &(i->modifyDate));
@@ -748,7 +765,7 @@ ncp_find_dir_inode(struct inode *dir, const char *name)
 
         do
 	{
-		if (   (result->dir->finfo.i.DosDirNum == dir_info->DosDirNum)
+		if (   (result->dir->finfo.i.dirEntNum == dir_info->dirEntNum)
 		    && (result->dir->finfo.i.volNumber == dir_info->volNumber)
 		    && (strcmp(result->finfo.i.entryName, name) == 0)
 		    /* The root dir is never looked up using this
@@ -776,7 +793,7 @@ ncp_lookup(struct inode *dir, const char *__name, int len,
 	struct ncp_server *server;
         struct ncp_inode_info *result_info;
         int found_in_cache;
-
+	int down_case = 0;
 	char name[len+1];
 
 	*result = NULL;
@@ -887,20 +904,26 @@ ncp_lookup(struct inode *dir, const char *__name, int len,
         if (found_in_cache == 0)
 	{
 		int res;
-		str_upper(name);
 
 		DDPRINTK("ncp_lookup: do_lookup on %s/%s\n",
 			 NCP_ISTRUCT(dir)->entryName, name);
 
 		if (ncp_is_server_root(dir))
 		{
+			str_upper(name);
+			down_case = 1;
 			res = ncp_lookup_volume(server, name, &(finfo.i));
 		}
 		else
 		{
+			if (!ncp_preserve_case(dir))
+			{
+				str_upper(name);
+				down_case = 1;
+			}
 			res = ncp_obtain_info(server,
 					      NCP_ISTRUCT(dir)->volNumber,
-					      NCP_ISTRUCT(dir)->DosDirNum,
+					      NCP_ISTRUCT(dir)->dirEntNum,
 					      name, &(finfo.i));
 		}
 		if (res != 0)
@@ -912,7 +935,11 @@ ncp_lookup(struct inode *dir, const char *__name, int len,
         }
 
 	finfo.opened = 0;
-	str_lower(finfo.i.entryName);
+
+	if (down_case != 0)
+	{
+		str_lower(finfo.i.entryName);
+	}
 
 	if (!(*result = ncp_iget(dir, &finfo)))
 	{
@@ -949,7 +976,11 @@ ncp_create(struct inode *dir, const char *name, int len, int mode,
 
 	strncpy(_name, name, len);
 	_name[len] = '\0';
-	str_upper(_name);
+
+	if (!ncp_preserve_case(dir))
+	{
+		str_upper(_name);
+	}
 
 	lock_super(dir->i_sb);
 	if (ncp_open_create_file_or_subdir(NCP_SERVER(dir),
@@ -966,7 +997,11 @@ ncp_create(struct inode *dir, const char *name, int len, int mode,
 
         ncp_invalid_dir_cache(dir);
 
-	str_lower(finfo.i.entryName);
+	if (!ncp_preserve_case(dir))
+	{
+		str_lower(finfo.i.entryName);
+	}
+
 	finfo.access = O_RDWR;
 
 	if (!(*result = ncp_iget(dir, &finfo)) < 0)
@@ -1000,7 +1035,11 @@ ncp_mkdir(struct inode *dir, const char *name, int len, int mode)
 
 	strncpy(_name, name, len);
 	_name[len] = '\0';
-	str_upper(_name);
+
+	if (!ncp_preserve_case(dir))
+	{
+		str_upper(_name);
+	}
 
 	if (!dir || !S_ISDIR(dir->i_mode))
 	{
@@ -1058,7 +1097,11 @@ ncp_rmdir(struct inode *dir, const char *name, int len)
 
 		strncpy(_name, name, len);
 		_name[len] = '\0';
-		str_upper(_name);
+
+		if (!ncp_preserve_case(dir))
+		{
+			str_upper(_name);
+		}
 
                 if ((error = ncp_del_file_or_subdir(NCP_SERVER(dir),
 						    NCP_ISTRUCT(dir),
@@ -1101,7 +1144,11 @@ ncp_unlink(struct inode *dir, const char *name, int len)
 	{
 		strncpy(_name, name, len);
 		_name[len] = '\0';
-		str_upper(_name);
+
+		if (!ncp_preserve_case(dir))
+		{
+			str_upper(_name);
+		}
 
                 if ((error = ncp_del_file_or_subdir(NCP_SERVER(dir),
 						    NCP_ISTRUCT(dir),
@@ -1155,11 +1202,19 @@ ncp_rename(struct inode *old_dir, const char *old_name, int old_len,
 
 	strncpy(_old_name, old_name, old_len);
 	_old_name[old_len] = '\0';
-	str_upper(_old_name);
+
+	if (!ncp_preserve_case(old_dir))
+	{
+		str_upper(_old_name);
+	}
 
 	strncpy(_new_name, new_name, new_len);
 	_new_name[new_len] = '\0';
-	str_upper(_new_name);
+
+	if (!ncp_preserve_case(new_dir))
+	{
+		str_upper(_new_name);
+	}
 
 	res = ncp_ren_or_mov_file_or_subdir(NCP_SERVER(old_dir),
 					    NCP_ISTRUCT(old_dir), _old_name,
