@@ -1149,6 +1149,11 @@ long
 ncp_close(struct ncp_conn *conn)
 {
 	long result;
+	if (conn == NULL)
+	{
+		return 0;
+	}
+
 	if ((result = ncp_do_close(conn)) != 0)
 	{
 		return result;
@@ -1292,7 +1297,7 @@ ncp_get_nwc_ent(FILE *nwc)
 }
 
 FILE *
-ncp_fopen_nwc(const char *user, const char *mode)
+ncp_fopen_nwc(const char *user, const char *mode, long *err)
 {
 	char path[MAXPATHLEN];
 	char *home = NULL;
@@ -1320,7 +1325,7 @@ ncp_fopen_nwc(const char *user, const char *mode)
 	if (   (home == NULL)
 	    || (strlen(home) + sizeof(NWCLIENT) + 2  > sizeof(path)))
 	{
-		errno = ENAMETOOLONG;
+		*err = ENAMETOOLONG;
 		return NULL;
 	}
 
@@ -1330,12 +1335,13 @@ ncp_fopen_nwc(const char *user, const char *mode)
 
 	if (stat(path, &st) != 0)
 	{
+		*err = errno;
 		return NULL;
 	}
 
 	if ((st.st_mode & (S_IRWXO | S_IRWXG)) != 0)
 	{
-		errno = EINVAL;
+		*err = NCPL_ET_INVALID_MODE;
 		return NULL;
 	}
 	
@@ -1371,11 +1377,9 @@ ncp_find_conn_spec(const char *server, const char *user, const char *password,
 	}
 	else
 	{
-		nwc = ncp_fopen_nwc(NULL, NULL);
-
-		if (nwc == NULL)
+		if ((nwc = ncp_fopen_nwc(NULL, NULL, err)) == NULL)
 		{
-			*err = NCPL_ET_NO_SPEC;
+			*err = NCPL_ET_NO_SERVER;
 			return NULL;
 		}
 
@@ -1391,6 +1395,15 @@ ncp_find_conn_spec(const char *server, const char *user, const char *password,
 		strcpy(spec.user, nwc_ent->user);
 	}
 
+	str_upper(spec.server);
+
+	if (login_necessary == 0)
+	{
+		memset(spec.user, 0, sizeof(spec.user));
+		memset(spec.password, 0, sizeof(spec.password));
+		return &spec;
+	}
+
 	if (user != NULL)
 	{
 		if (strlen(user) >= sizeof(spec.user))
@@ -1401,7 +1414,6 @@ ncp_find_conn_spec(const char *server, const char *user, const char *password,
 		strcpy(spec.user, user);
 	}
 
-	str_upper(spec.server);
 	str_upper(spec.user);
 	spec.login_type = NCP_BINDERY_USER;
 
@@ -1422,7 +1434,7 @@ ncp_find_conn_spec(const char *server, const char *user, const char *password,
 	}
 	else
 	{
-		if ((nwc = ncp_fopen_nwc(NULL, NULL)) != NULL)
+		if ((nwc = ncp_fopen_nwc(NULL, NULL, err)) != NULL)
 		{
 			while ((nwc_ent = ncp_get_nwc_ent(nwc)) != NULL)
 			{
@@ -1442,15 +1454,10 @@ ncp_find_conn_spec(const char *server, const char *user, const char *password,
 		}
 	}
 
-	if (login_necessary == 0)
-	{
-		memset(spec.user, 0, sizeof(spec.user));
-		memset(spec.password, 0, sizeof(spec.password));
-	}
-
 	if (strlen(spec.user) == 0)
 	{
-		return &spec;
+		*err = NCPL_ET_NO_USER;
+		return NULL;
 	}
 
 	if ((strlen(spec.password) == 0) && (password == NULL))
@@ -1817,6 +1824,31 @@ ncp_get_file_server_time(struct ncp_conn *conn, time_t *target)
 	return 0;
 }
 
+long
+ncp_get_file_server_information(struct ncp_conn *conn,
+				struct ncp_file_server_info *target)
+{
+	long result;
+	ncp_init_request_s(conn, 17);
+	if ((result = ncp_request(conn, 23)) != 0)
+	{
+		ncp_unlock_conn(conn);
+		return result;
+	}
+
+	memcpy(target, ncp_reply_data(conn, 0), sizeof(*target));
+	target->MaximumServiceConnections
+		= htons(target->MaximumServiceConnections);
+	target->ConnectionsInUse
+		= htons(target->ConnectionsInUse);
+	target->MaxConnectionsEverUsed
+		= htons(target->MaxConnectionsEverUsed);
+	target->NumberMountedVolumes
+		= htons(target->NumberMountedVolumes);
+	ncp_unlock_conn(conn);
+	return 0;
+}
+
 long
 ncp_get_connlist(struct ncp_conn *conn,
 		 __u16 object_type, const char *object_name,
@@ -1840,6 +1872,57 @@ ncp_get_connlist(struct ncp_conn *conn,
 	return 0;
 }
 
+long
+ncp_get_stations_logged_info(struct ncp_conn *conn,
+			     __u32 connection,
+			     struct ncp_bindery_object *target,
+			     time_t *login_time)
+{
+	long result;
+	ncp_init_request_s(conn, 28);
+	ncp_add_dword(conn, connection);
+
+	if ((result = ncp_request(conn, 23)) != 0)
+	{
+		ncp_unlock_conn(conn);
+		return result;
+	}
+	memset(target, 0, sizeof(*target));
+	target->object_id = ntohl(ncp_reply_dword(conn, 0));
+	target->object_type = ntohs(ncp_reply_word(conn, 4));
+	memcpy(target->object_name, ncp_reply_data(conn, 6),
+	       sizeof(target->object_name));
+	*login_time = nw_to_ctime((struct nw_time_buffer *)
+				  ncp_reply_data(conn, 54));
+	ncp_unlock_conn(conn);
+	return 0;
+}
+
+long
+ncp_get_internet_address(struct ncp_conn *conn,
+			 __u32 connection,
+			 struct sockaddr_ipx *target,
+			 __u8 *conn_type)
+{
+	long result;
+	ncp_init_request_s(conn, 26);
+	ncp_add_dword(conn, connection);
+
+	if ((result = ncp_request(conn, 23)) != 0)
+	{
+		ncp_unlock_conn(conn);
+		return result;
+	}
+
+	memset(target, 0, sizeof(*target));
+	target->sipx_network = ncp_reply_dword(conn, 0);
+	memcpy(&(target->sipx_node), ncp_reply_data(conn, 4), 6);
+	target->sipx_port = ncp_reply_word(conn, 10);
+	*conn_type = ncp_reply_byte(conn, 12);
+	ncp_unlock_conn(conn);
+	return 0;
+}
+
 long
 ncp_send_broadcast(struct ncp_conn *conn,
 		   __u8 no_conn, const __u8 *connections,
@@ -2196,6 +2279,21 @@ ncp_login_encrypted(struct ncp_conn *conn,
 	return result;
 }
 
+long
+ncp_login_unencrypted(struct ncp_conn *conn,
+		      __u16 object_type, const char *object_name,
+		      const unsigned char *passwd)
+{
+	long result;
+	ncp_init_request_s(conn, 20);
+	ncp_add_word(conn, htons(object_type));
+	ncp_add_pstring(conn, object_name);
+	ncp_add_pstring(conn, passwd);
+	result = ncp_request(conn, 23);
+	ncp_unlock_conn(conn);
+	return result;
+}
+
 long
 ncp_change_login_passwd(struct ncp_conn *conn,
 			const struct ncp_bindery_object *object,
@@ -2252,7 +2350,8 @@ ncp_login_object(struct ncp_conn *conn,
 	struct ncp_bindery_object user;
 
 	if ((result = ncp_get_encryption_key(conn, ncp_key)) != 0) {
-		return result;
+		return ncp_login_unencrypted(conn, login_type, username,
+					     password);
 	}
 
 	if ((result = ncp_get_bindery_object_id(conn, login_type,
@@ -2672,7 +2771,7 @@ static void
 ncp_add_handle_path(struct ncp_conn *conn,
 		    __u8 vol_num,
 		    __u32 dir_base, int have_dir_base,
-		    char *path)
+		    const char *path)
 {
 	ncp_add_byte(conn, vol_num);
 	ncp_add_dword(conn, dir_base);
@@ -2705,6 +2804,33 @@ ncp_extract_file_info(void *structure, struct nw_info_struct *target)
 	return;
 }
 
+long
+ncp_obtain_file_or_subdir_info(struct ncp_conn *conn,
+			       __u8 source_ns, __u8 target_ns,
+			       __u16 search_attribs, __u32 rim,
+			       __u8 vol, __u32 dirent, const char *path,
+			       struct nw_info_struct *target)
+{
+	long result;
+
+	ncp_init_request(conn);
+	ncp_add_byte(conn, 6);
+	ncp_add_byte(conn, source_ns);
+	ncp_add_byte(conn, target_ns);
+	ncp_add_word(conn, search_attribs);
+	ncp_add_dword(conn, rim);
+	ncp_add_handle_path(conn, vol, dirent, 1, path);
+
+	if ((result = ncp_request(conn, 87)) != 0)
+	{
+		ncp_unlock_conn(conn);
+		return result;
+	}
+	ncp_extract_file_info(ncp_reply_data(conn, 0), target);
+	ncp_unlock_conn(conn);
+	return 0;
+}	
+
 long
 ncp_do_lookup(struct ncp_conn *conn,
 	      struct nw_info_struct *dir,
@@ -2893,7 +3019,7 @@ ncp_initialize_search(struct ncp_conn *conn,
 		return result;
 	}
 
-	memcpy(&(target->s), ncp_reply_data(conn, 0), sizeof(target->s));
+	memcpy(&(target->s), ncp_reply_data(conn, 0), 9);
 	target->namespace = namespace;
 
 	ncp_unlock_conn(conn);
