@@ -155,11 +155,48 @@ ncp_get_file_server_description_strings(struct ncp_conn *conn,
 long
 ncp_get_file_server_time(struct ncp_conn *conn, time_t *target);
 
+struct ncp_file_server_info {
+	__u8	ServerName[48]            __attribute__ ((packed));
+	__u8    FileServiceVersion        __attribute__ ((packed));
+	__u8    FileServiceSubVersion     __attribute__ ((packed));
+	__u16	MaximumServiceConnections __attribute__ ((packed));
+	__u16	ConnectionsInUse          __attribute__ ((packed));
+	__u16	NumberMountedVolumes      __attribute__ ((packed));
+	__u8	Revision                  __attribute__ ((packed));
+	__u8	SFTLevel                  __attribute__ ((packed));
+	__u8	TTSLevel                  __attribute__ ((packed));
+	__u16	MaxConnectionsEverUsed    __attribute__ ((packed));
+	__u8	AccountVersion            __attribute__ ((packed));
+	__u8	VAPVersion                __attribute__ ((packed));
+	__u8	QueueVersion              __attribute__ ((packed));
+	__u8	PrintVersion              __attribute__ ((packed));
+	__u8	VirtualConsoleVersion     __attribute__ ((packed));
+	__u8	RestrictionLevel          __attribute__ ((packed));
+	__u8	InternetBridge            __attribute__ ((packed));
+	__u8	Reserved[60]              __attribute__ ((packed));
+};
+
+long
+ncp_get_file_server_information(struct ncp_conn *conn,
+				struct ncp_file_server_info *target);
+
 long
 ncp_get_connlist(struct ncp_conn *conn,
 		 __u16 object_type, const char *object_name,
 		 int *returned_no, __u8 conn_numbers[256]);
 
+long
+ncp_get_stations_logged_info(struct ncp_conn *conn,
+			     __u32 connection,
+			     struct ncp_bindery_object *target,
+			     time_t *login_time);
+
+long
+ncp_get_internet_address(struct ncp_conn *conn,
+			 __u32 connection,
+			 struct sockaddr_ipx *target,
+			 __u8  *conn_type);
+
 long
 ncp_send_broadcast(struct ncp_conn *conn,
 		   __u8 no_conn, const __u8 *connections,
@@ -198,6 +235,12 @@ ncp_change_object_security(struct ncp_conn *conn,
 			   const char *object_name,
 			   __u8 security);
 
+struct ncp_station_addr {
+    __u32   NetWork __attribute__ ((packed));
+    __u8    Node[6] __attribute__ ((packed));
+    __u16   Socket  __attribute__ ((packed));
+};
+
 struct ncp_prop_login_control {
     __u8    AccountExpireDate[3]     __attribute__ ((packed));
     __u8    Disabled                 __attribute__ ((packed));
@@ -214,7 +257,7 @@ struct ncp_prop_login_control {
     __u32   MaxDiskUsage             __attribute__ ((packed));
     __u16   BadLoginCount            __attribute__ ((packed));
     __u32   BadLoginCountDown        __attribute__ ((packed));
-    __u8    LastIntruder[8]          __attribute__ ((packed));
+    struct ncp_station_addr LastIntruder  __attribute__ ((packed));
 };	
 
 long
@@ -265,6 +308,11 @@ ncp_login_encrypted(struct ncp_conn *conn,
 		    const unsigned char *key,
 		    const unsigned char *passwd);
 
+long
+ncp_login_unencrypted(struct ncp_conn *conn,
+		      __u16 object_type, const char *object_name,
+		      const unsigned char *passwd);
+
 long
 ncp_change_login_passwd(struct ncp_conn *conn,
 			const struct ncp_bindery_object *object,
@@ -373,6 +421,14 @@ ncp_copy_file(struct ncp_conn *conn,
 	      __u32 count,
 	      __u32 *copied_count);
 
+long
+ncp_obtain_file_or_subdir_info(struct ncp_conn *conn,
+			       __u8 source_ns, __u8 target_ns,
+			       __u16 search_attribs, __u32 rim,
+			       __u8 vol, __u32 dirent, const char *path,
+			       struct nw_info_struct *target);
+
+
 long
 ncp_do_lookup(struct ncp_conn *conn,
 	      struct nw_info_struct *dir,
