@@ -15,17 +15,18 @@
 int
 main(int argc, char **argv)
 {
-	struct ncp_conn conn;
+	struct ncp_conn *conn;
 	struct ncp_bindery_object q;
 	int found = 0;
 
 	char default_pattern[] = "*";
 	char *pattern = default_pattern;
 	char *p;
+	long err;
 
-	if (ncp_initialize(&conn, &argc, argv, 1) != 0)
+	if ((conn = ncp_initialize(&argc, argv, 1, &err)) == NULL)
 	{
-		perror("Could not open connection");
+		com_err(argv[0], err, "when initializing");
 		return 1;
 	}
 
@@ -47,7 +48,7 @@ main(int argc, char **argv)
 
 	if (isatty(1))
 	{
-		printf("\nServer: %s\n", conn.server);
+		printf("\nServer: %s\n", conn->server);
 		printf("%-52s%-10s\n"
 		       "-----------------------------------------------"
 		       "-------------\n",
@@ -57,8 +58,8 @@ main(int argc, char **argv)
 
 	q.object_id = 0xffffffff;
 
-	while (ncp_scan_bindery_object(&conn, q.object_id,
-				       NCP_BINDERY_PQUEUE, "*", &q) == 0)
+	while (ncp_scan_bindery_object(conn, q.object_id,
+				       NCP_BINDERY_PQUEUE, pattern, &q) == 0)
 	{
 		found = 1;
 		printf("%-52s", q.object_name);
@@ -70,6 +71,6 @@ main(int argc, char **argv)
 		printf("No queues found\n");
 	}
 
-	ncp_close(&conn);
+	ncp_close(conn);
 	return 0;
 }
